@@ -47,7 +47,7 @@
 
 
 
-randomPoints <- function(mask, n, p, ext=NULL, extf=1.1, excludep=TRUE, tryf=5, warn=2) {
+randomPoints <- function(mask, n, p, ext=NULL, extf=1.1, excludep=TRUE, cellnumbers=FALSE, tryf=5, warn=2) {
 	
 	if (nlayers(mask) > 1) { mask <- raster(mask, 1)	}
 	
@@ -92,8 +92,10 @@ randomPoints <- function(mask, n, p, ext=NULL, extf=1.1, excludep=TRUE, tryf=5,
 	if (canProcessInMemory(mask2)) {
 	
 		cells <- crop(mask, mask2)
-		cells <- which(! is.na(getValues(cells)) )
-		nn = min(length(cells), nn)
+		if (hasValues(cells)) {
+			cells <- which(! is.na(getValues(cells)) )
+		}
+		nn <- min(length(cells), nn)
 
 		if (raster:::.couldBeLonLat(mask)) {
 			# which rows are that?
@@ -112,7 +114,7 @@ randomPoints <- function(mask, n, p, ext=NULL, extf=1.1, excludep=TRUE, tryf=5,
 	
 	} else {
 	
-		nn = min(ncell(mask2), nn)
+		nn <- min(ncell(mask2), nn)
 		if (raster:::.couldBeLonLat(mask)) {
 	
 			cells <- .randomCellsLonLat(mask2, nn)
@@ -128,8 +130,10 @@ randomPoints <- function(mask, n, p, ext=NULL, extf=1.1, excludep=TRUE, tryf=5,
 			cells <- cellFromXY(mask, xy)
 		}
 
-		vals <- cbind(cells, extract(mask, cells))
-		cells <- na.omit(vals)[,1]
+		if (hasValues(mask)) {
+			vals <- cbind(cells, extract(mask, cells))
+			cells <- na.omit(vals)[,1]
+		}
 	}
 		
 	if (excludep) {	
@@ -154,7 +158,11 @@ randomPoints <- function(mask, n, p, ext=NULL, extf=1.1, excludep=TRUE, tryf=5,
 		}
 	}
 	
-	return(xyFromCell(mask, cells))
+	if (cellnumbers) {
+		return(cells)
+	} else {
+		return(xyFromCell(mask, cells))
+	}
 }
 
 
